(function() {

    "use strict";

    // レコード追加画面／編集画面表示、レコード一覧画面のインライン編集開始
    // レコード保存のイベントは3種類あることに注意
    var eventsSubmit = [
        'app.record.create.submit',
        'app.record.edit.submit',
        'app.record.index.edit.submit'
    ];

    kintone.events.on(eventsSubmit, function(event) {

        var record = event.record;

        // 売上集計用のオブジェクトを初期化
        var reportTable = {};

        // 「開始日付」と「終了日付」の間を検索条件とする
        var queryCondition = '日付 >= "' + record['開始日付'].value + '" and 日付 <= "' + record['終了日付'].value + '"';
        var params = {
            "app": 52,
            "query": queryCondition
        };

        // 「6章日報」アプリを検索する
        return kintone.api(kintone.api.url('/k/v1/records', true), 'GET', params).then(function(resp) {
            // 成功時
            // 「6章日報」アプリの「売上明細」テーブルオブジェクトを取得する
            for (var i = 0; i < resp.records.length; i++) {
                // 「6章日報」アプリのテーブル内の1行を取得する
                var nippoTable = resp.records[i].売上明細.value;
                for (var j = 0; j < nippoTable.length; j++) {
                    if (!reportTable[nippoTable[j].value['商品名'].value]) {
                        // 売上集計用のオブジェクトがundefinedの場合は追加する
                        reportTable[nippoTable[j].value['商品名'].value] =
                            Number(nippoTable[j].value['売上金額'].value);
                    } else {
                        // 売上集計用のオブジェクトの商品名と同じものがあればその売上金額に加算する
                        reportTable[nippoTable[j].value['商品名'].value] =
                            Number(reportTable[nippoTable[j].value['商品名'].value]) +
                            Number(nippoTable[j].value['売上金額'].value);
                    }
                }
            }
            var tableRecords = [];
            for (var syohinName in reportTable) {
                if (reportTable.hasOwnProperty(syohinName)) {
                    tableRecords.push(
                        {
                            "value": {
                                "商品名": {
                                    "value": syohinName,
                                    "type": "SINGLE_LINE_TEXT"
                                },
                                "売上金額": {
                                    "value": reportTable[syohinName],
                                    "type": "NUMBER"
                                }
                            }
                        }
                    );
                }
            }
            // 「6章_売上レポート」アプリの「売上明細」テーブルに上書き
            record['売上明細']['value'] = tableRecords;
            return event;
        }, function(resp) {
            //error:エラーの場合はメッセージを表示する
            var errmsg = '「6章日報」アプリのレコード取得時にエラーが発生しました。';
            // レスポンスにエラーメッセージが含まれる場合はメッセージを表示する
            if (resp.message !== undefined) {
                errmsg += '\n' + resp.message;
            }
            alert(errmsg);
        });
    });
})();
